/* Calculator3-komponentin luokkaosa

Luokassa luodaan reaktiivinen lomake, jossa on neljä elementtiä. Koodi on melko 
pitkä, koska lomake-elementtien arvot tallennetaan myös luokan muuttujiin. 
Koodi olisi lyhyempi, jos luokan muuttujia ei käytettäisi, mutta arkkitehtuuri 
olisi tällöin huonompi. Nyt laskutoimitukset ovat omissa funktioissaan.
*/
import { Component, OnInit } from '@angular/core';
// Nämä tulevat ReactiveFormsModulesta
import { FormBuilder, FormGroup } from '@angular/forms';

@Component({
  selector: 'app-calculator3',
  templateUrl: './calculator3.component.html',
  styleUrls: ['./calculator3.component.css'],
})
export class Calculator3Component implements OnInit {
  
  //ominaisuudet eli luokan muuttujat
  val1: number;
  val2: number;
  result: number;
  op: string;
  form1!: FormGroup;

  /* Konstruktori antaa luokasta syntyvän olion muuttujille alkuarvot
     Konstruktorin argumenttina on luotu FormBuilder-olio fb, jolla voidaan
     yksinkertaistaa reaktiivisten lomakkeiden luontia.
  */
  constructor(private fb: FormBuilder) {
    // luokan muuttujien alkuarvot
    this.val1 = 0;
    this.val2 = 0;
    this.op = '+';
    this.result = 0;
  }

  // metodit

  /* ngOnInit()-metodissa alustetaan reaktiivinen lomake, joka syntyy
     tässä tapauksessa FormBuilder-tyyppisestä fb-oliosta.
  */
  ngOnInit(): void {
    this.form1 = this.fb.group({
      // Nämä nimet viittaavat lomakkeen elementteihin (formControlName).
      // Eivät siis ole luokan muuttujia, vaikka samoja nimiä käytetäänkin.
      val1: 0,
      val2: 0,
      op: '+',
      result: 0,
    });
    // onChanges() -metodilla tarkkaillaan lomakkeella tapahtuvia muutoksia
    this.onChanges();
  }
  
  onChanges(): void {
    this.getValue('val1');
    this.getValue('val2');
    this.getValue('op');
  }
  // Haetaan lomakkeen arvojen muutokset ja välitetään ne setResult-metodille
  getValue(val: string){
    this.form1.get(val)!.valueChanges.subscribe((res: any) => {
      this.setResult();
    });
  }

  setResult() {
  // Vastaanotetaan lomakkeen arvojen muutokset ja laitetaan arvot
  // luokan muuttujiin
    this.val1 = this.form1.get('val1')!.value;
    this.val2 = this.form1.get('val2')!.value;
    this.op = this.form1.get('op')!.value;

    if (this.op === '+') {
      this.doPlus();
    } else if (this.op === '-') {
      this.doMinus();
    } else if (this.op === '*') {
      this.doMulti();
    }
    // laitetaan tulos lomakkeen result-elementtiin
    this.form1.get('result')!.setValue(this.result);
  }

  // Sovelluslogiikka eli laskutoimitukset
  public doPlus(){
    this.result = this.val1 + this.val2;
  }

  public doMinus(){
    this.result = this.val1 - this.val2;
  }

  public doMulti(){
    this.result = this.val1 * this.val2;
  }
}
